/*!
	\file	sipClientModule.cc
	\author	jAN bURGHARD
	\version 0.2
	
*/
#include "sipClientModule.h"


Define_Module(sipClientModule);

//
// methods inherited from cSimpleModule
//


void sipClientModule::initialize() {

	// initialize variables
	_debug = par("debug");
	
};



void sipClientModule::handleMessage(cMessage *msg){
	if (msg->arrivalGateId() == findGate("fromApp")) {
		_handleMessageFromApp( msg );
	} 

	else if (msg->arrivalGateId() == findGate("fromClientTrans")) {
		_handleMessageFromClientTransaction( msg );
	} 

	else {
		opp_error("sipClientModule: Message from unknown Gate.");
	}
	delete msg;
};	



void sipClientModule::_handleMessageFromApp(cMessage *msg) {
	/* either we've to forward a sipRequest or a transaction should be canceled */
	/* in every case we expect "sipInterfacePackets"                            */

	sipInterfacePacket *sIfPkt = (sipInterfacePacket *)msg;
	if ( _debug ) myEv << "sipClientModule : received msg -> " 
		<< sIfPkt->name() << endl;

	// needed objects
	sipInterfacePacket *sIfPktOut;
	sipRequest *request;
	cModule *mod;
	
	
	switch ( sIfPkt->action() ) {
	
	case sipInterfacePacket::SA_REQUEST:
		request = sIfPkt->getRequest();
	
		if (simulation.module(findSubmodule( request->requestKey() )) == NULL) {
			if ( _debug ) myEv << "creating new clientlayer: " 
				<< request->requestKey() << endl;
			// session with given sessionId does not 
			// exist -> create new session
			_createTransaction( request, sipInterfacePacket::SA_REQUEST );
		}

		else {
			// forward message to submodule
			mod = simulation.module(findSubmodule( request->requestKey() ));
			if ( _debug ) myEv << "forwarding message to transaction: " 
				<< mod->name() << endl;
			sIfPktOut = new sipInterfacePacket("FWD_FROM_APP");
			sIfPktOut->request( request );
			sendDirect( sIfPktOut, 0.0, mod, "fromApp" );		
		}
		break;

	case sipInterfacePacket::SA_ACK:
        
		request = sIfPkt->getRequest();

		// handle special ACK case.
		if ( _debug ) myEv << "creating new clientlayer: " 
			<< request->requestKey() << endl;
		_createTransaction( request, sipInterfacePacket::SA_ACK );
		break;
		
	default:
		opp_error("unsupported sip action");
		break;
	}
};


/*

*/
void sipClientModule::_handleMessageFromClientTransaction(cMessage *msg) {
	// we need sipInterfacePackets
	sipInterfacePacket *sIfPkt = (sipInterfacePacket *)msg;

	// needed objects
	sipInterfacePacket *sIfPktOut;
	sipResponse *response;
	cModule *mod;

	switch ( sIfPkt->action() ) {
	
	case sipInterfacePacket::SA_CLOSE_SESSION:
		if (_debug) myEv << "deleting ClientModule"
			<< simulation.module(msg->senderModuleId())->name() 
			<< endl;
		// delete dynamically submodule		
		mod = simulation.module(msg->senderModuleId());
		mod->deleteModule();
		break;
	
	
	// inform TU about timeouts
	case sipInterfacePacket::SA_TIMEOUT_REQUEST:
		sIfPktOut = sIfPkt->dup();
		send( sIfPktOut, "toApp" );
		break;
		
			
	default:
		if (_debug) myEv << "forwarding Message to Application" << endl;
		// default is to forward the Message to the Application
		response = sIfPkt->getResponse();
		sIfPktOut = new sipInterfacePacket( sIfPkt->name() );
		sIfPktOut->respond( response );
		send( sIfPktOut, "toApp" );
	}
};

	
	
void sipClientModule::_createTransaction(sipRequest *request, sipInterfacePacket::sipAction action) {
/* 
	This code was taken from Arndt Buschmann's RTSP code - thanks
*/

	// find and create submodule
	cModuleType *moduleType = findModuleType("sipClientTransaction");
	if ( moduleType == NULL ) {
		opp_error("Module 'sipClientTransaction' not found!");
		return;
	}

	// create the submodule-name
	cModule *mod = moduleType->create(request->requestKey(), this);

	// set parameters
	mod->par("debug") = par("debug");
	
	moduleType->buildInside( mod );
	mod->callInitialize();
	mod->scheduleStart( simTime() );

	// find SocketLayer Module
	cModule *socketMod = gate("toSocketLayer")->toGate()->ownerModule();

	// find free gate at SocketLayer
	int j = 0;
	while (socketMod->gate("from_appl", j)->isConnected()) {
		j++;
		if (socketMod->gate("from_appl")->size() == j) 
			opp_error("SocketLayer gateSize() to small.");
	}
	
	if (_debug) myEv << "connecting mod '" << mod->name() 
		<< "' to gates no '" << j << "' of mod '" 
		<< socketMod->name() << "'" << endl;
		
	// set links from and to SocketLayer
	mod->gate("toSocketLayer") -> setTo(socketMod->gate("from_appl", j));
	mod->gate("fromSocketLayer") -> setFrom(socketMod->gate("to_appl", j));
	socketMod->gate("from_appl", j)-> setFrom(mod->gate("toSocketLayer"));
	socketMod->gate("to_appl", j) -> setTo(mod->gate("fromSocketLayer"));
	
	
	for (int i=1; i <= socketMod->gate("from_appl")->size(); i++) {
		if (socketMod->gate("from_appl",i-1)->isConnected() && _debug)
			ev << "Gate " << i-1 << "is connected. " << endl;
	}

	sipInterfacePacket *sipIfPkt = new sipInterfacePacket("NEW TRANSACTION");
	switch ( action ) {
	case sipInterfacePacket::SA_REQUEST:
		sipIfPkt->request( request );
		break;
	case sipInterfacePacket::SA_ACK:
		sipIfPkt->ack( request );
		break;
	default:
		opp_error("unknown SIP-Action!");
		break;
	}
	
	// forward request
	sendDirect(sipIfPkt, 0.0, mod, "fromApp");

};
