/*!
	\file	sipClientModule.h
	\author	jAN bURGHARD
	\version 0.2
	
*/
#ifndef __SIPCLIENTMODULE_H
#define __SIPCLIENTMODULE_H

// for gcc3.x
using namespace std;

// OMNeT++
#include <omnetpp.h>

// SIP-stuff
#include "../Messages/sipConstants.h"
#include "../Messages/sipRequest.h"
#include "../Messages/sipInterfacePacket.h"


//! SIP client-module
/*!
	\class sipClientModule sipClientModule.h
	This module manages creation, destruction and operation of a
	SIP-hosts client transactions and the communication between 
	SIP-applications and SIP-transactions

	\author Jan Burghard
	\sa sipClientTransaction
*/
class sipClientModule : public cSimpleModule {

private:
	//! debugging on or off?
	int _debug;
		

protected:
	//! Handles messages received from the application.
	/*!
	This method tries to match the message (from the sipApplication)
	to an existing sipClientTransaction. If none is found a new will
	be created. Than the message will be passed over to that transaction.
	\param msg a sipInterfacePacket as cMessage
	*/
	virtual void _handleMessageFromApp(cMessage *msg);

	//! Handles messages received from client process.
	/*!
	Casts the message to a sipInterfacePacket and decides with its
	action()-value what to do: forwarding an encapsulated sipResponse
	to the application or delete the sending transaction.
	\param msg a sipInterfacePacket as cMessage	
	*/
	virtual void _handleMessageFromClientTransaction(cMessage *msg);
			
	//! Creates a new sipTransaction.
	/*!
	This includes moduletype computation, module creation, parameter
	passing, module initialization and connecting the gates.
	\param request sipRequest to be sent to the new transaction
	\param action sipAction to determine if the request shall be handeled 
	  as normal request (SA_REQUEST) or if it should just be sent (SA_ACK).
	*/
	virtual void _createTransaction(sipRequest *request, sipInterfacePacket::sipAction action);

public:
	
	//! OMNeT++ macro
	/*!
	will create constructors and so on.
	*/
	Module_Class_Members(sipClientModule, cSimpleModule, 0);
	
	//! inherited from OMNeT++
	/*!
	initializes the class.
	*/
	virtual void initialize();
	
	//! inherited from OMNeT++
	/*!
	handles incoming messages.
	\param msg the cMessage to handle
	*/
	virtual void handleMessage(cMessage *msg);
};

#endif
