/*!
	\file	sipConstants.h
	\author jAN bURGHARD
	\version 0.1
	
	In this file all constant definitions sould be made.
*/

#ifndef __SIPCONSTANTS_H
#define __SIPCONSTANTS_H


/* --------------------------------------------------------------
      Constants
   -------------------------------------------------------------- */

//! SIP magic cookie
#define SIP_MAGIC_COOKIE "z9hG4bK"
// const char *SIP_MAGIC_COOKIE = "z9hG4bK";

// constants used in cMessage's "kind" field
// makes the messages appear in different colors
const int MK_SIPREQUEST		= 1000;
const int MK_SIPRESPONSE	= 1001;
const int MK_SIPINTERFACEPACKET	= 1002;
const int MK_SIPINNERPACKET	= 1003;

/*! 
Value to be added when recording the "action()"-value of a sipInterfacePacket.
With this value added it is possible to differ between request-methods (values
lower than 50), sipInterfacePacket-actions (values between 50 and 100) and
response-codes (values from 100 to 699).
*/
const int REC_SIPINTERFACE_ACTION = 50;


// how long is an ip-address
const int IP_DIGIT_COUNT = 15;



// constants for headerfields
const int MAX_ALLOW = 6;
const int MAX_SUPPORTED = 5;
const int MAX_ACCEPT = 2; // SDP
const int MAX_CONTENT_DISPOSITION = 2; // "session" or "render"
const int MAX_CONTENT_LANGUAGE = 2; // not implemented
const int MAX_CONTENT_ENCODING = 2; // not implemented
const int MAX_EXTENSION = 2; // not implemented
const int MAX_METHOD = 6;



//! possible transport protocols
enum enProtocols { 
	UDP, 
	TCP, 
	TLS 
};


//! SIP request methods
enum sipRequestMethod {
	UNDEF		= 0,
	INVITE		= 1,
	ACK		= 2,
	OPTIONS		= 3,
	BYE		= 4,
	CANCEL		= 5,
	REGISTER	= 6
};
//! strings to the sipRequestMethods
static char *const sipRequestMethodStr[] = {
	"UNDEFINED",
	"INVITE",
	"ACK",
	"OPTIONS",
	"BYE",
	"CANCEL",
	"REGISTER"
};
	
	



//! Call-ID structure
struct stCallId {
	char *id;
	char *host;
};

//! CSeq structure
struct stCSeq {
	int seqNumber;
	sipRequestMethod method;
};



//! extended ev-operator
#define myEv ev << simtimeToStr( simTime() ) << " " << name() << " (" << id() << ") "

#endif
