/*!
	\file	sipHeader.cc
	\author	jAN bURGHARD
	\author Susann Geisler
	\version 0.4
*/
#include "sipHeader.h"


// default constructor
sipHeader::sipHeader() 
{ 
	_init();
};

// copy constructor
sipHeader::sipHeader(const sipHeader& h)
{
	_init();
	operator=( h );
};

// default destructor
sipHeader::~sipHeader() {
	while ( !_via.empty() ) {
		delete _via.front();
		_via.pop_front();
	}
	while ( !_route.empty() ) {
		delete _route.front();
		_route.pop_front();
	}
/*	while ( !_recordRoute.empty() ) {
		delete _recordRoute.front();
		_recordRoute.pop_front();
	}
*/
	
	while ( !_contact.empty() ) {
		delete _contact.front();
		_contact.erase( _contact.begin() );
	}

	delete _contentType;
	delete _require;
	delete _callId.id;
	delete _callId.host;
};


// = operator
sipHeader& sipHeader::operator=(const sipHeader& h) {
	_from		= h._from;
	_to		= h._to;
	_callId.id	= opp_strdup( h._callId.id );
	_callId.host	= opp_strdup( h._callId.host );
	_cSeq		= h._cSeq;
	_contentType	= opp_strdup( h._contentType );
	_contentLength	= h._contentLength;
	_require	= opp_strdup( h._require );

	// the pointers will be copied correctly - but the objects NOT!
	// therefore we do it ourselfs
	
	setViaList( h._via );	
	setRouteList( h._route );
	
	
/*	_via		= h._via;
	list<sipTagVia *>::iterator it_via = _via.begin();
	while ( it_via != _via.end() ) {
		*it_via = new sipTagVia( **it_via );
		it_via++;
	}

	_route		= h._route;
	list<sipTagRoute *>::iterator it_rt = _route.begin();
	while ( it_rt != _route.end() ) {
		*it_rt = new sipTagRoute( **it_rt );
		it_rt++;
	}
	
	_recordRoute	= h._recordRoute;
	list<sipTagRecordRoute *>::iterator it_rrt = _recordRoute.begin();
	while ( it_rrt != _recordRoute.end() ) {
		*it_rrt = new sipTagRecordRoute( **it_rrt );
		it_rrt++;
	}
*/

	_contact.clear();
	setContacts( h._contact );

	_maxForwards	= h._maxForwards;

	for(int i = 0; i < MAX_ALLOW; i++)
		_allow[i] = h._allow[i];

	for(int i = 0; i < MAX_SUPPORTED; i++)
		_supported[i] = h._supported[i];

	for(int i = 0; i < MAX_ACCEPT; i++)
		_accept[i] = h._accept[i];

	for(int i = 0; i < MAX_CONTENT_DISPOSITION; i++)
		_contentDisposition[i] = h._contentDisposition[i];

	for(int i = 0; i < MAX_CONTENT_LANGUAGE; i++)
		_contentLanguage[i] = h._contentLanguage[i];

	for(int i = 0; i < MAX_CONTENT_ENCODING; i++)
		_contentEncoding[i] = h._contentEncoding[i];

	return *this;
}


void sipHeader::_init() {
//	sipAddress	_uri;
//	sipAddress	_from;
//	sipAddress	_to;
//	_via.clear();
//	sipAddress	vFrom;
//	sipAddress	vTo;
	_callId.id	= NULL;
	_callId.host	= NULL;
//	stCSeq		vCSeq; 
	_contentType	= NULL;
	_contentLength	= 0;
	_require	= NULL;

//	_route.clear();
//	_recordRoute.clear();

	_maxForwards	= -1;

	_allow = (int *)malloc(MAX_ALLOW * sizeof(int));
	for(int i = 0; i < MAX_ALLOW; i++)
		_allow[i] = 0;

	_supported = (int *)malloc(MAX_SUPPORTED * sizeof(int));
	for(int i = 0; i < MAX_SUPPORTED; i++)
		_supported[i] = 0;

	_accept = (int *)malloc(MAX_ACCEPT * sizeof(int));
	for(int i = 0; i < MAX_ACCEPT; i++)
		_accept[i] = 0;

	_contentDisposition = (int *)malloc(MAX_CONTENT_DISPOSITION * sizeof(int));
	for(int i = 0; i < MAX_CONTENT_DISPOSITION; i++)
		_contentDisposition[i] = 0;

	_contentLanguage = (int *)malloc(MAX_CONTENT_LANGUAGE * sizeof(int));
	for(int i = 0; i < MAX_CONTENT_LANGUAGE; i++)
		_contentLanguage[i] = 0;

	_contentEncoding = (int *)malloc(MAX_CONTENT_ENCODING * sizeof(int));
	for(int i = 0; i < MAX_CONTENT_ENCODING; i++)
		_contentEncoding[i] = 0;

};



// ------------
// Set-Methoden
// ------------


void sipHeader::setUri(sipAddress address) {
	_uri = address;
};

void sipHeader::setFrom(sipAddress address) {
	_from = address;
}

void sipHeader::setTo(sipAddress address) {
	_to = address;
}

void sipHeader::addContact(sipAddress *address) {
	_contact.push_back( address );
};

void sipHeader::setContacts(vector<sipAddress *> contacts) {
	_contact.clear();
	for(unsigned int i = 0; i < contacts.size(); i++) {
		_contact.push_back( new sipAddress(*contacts[i]) );
	}
};



void sipHeader::setCallId(stCallId s) {
	_callId.id   = opp_strdup( s.id );
	_callId.host = opp_strdup( s.host );
}

void sipHeader::setCallId(const char *id) {
	_callId.id = opp_strdup( id );
}

void sipHeader::setCSeq(stCSeq s) {
	_cSeq = s;
}

void sipHeader::setCSeq(sipRequestMethod method, const int seq) {
	_cSeq.method	= method;
	_cSeq.seqNumber	= seq;
};



/* adds a "Via"-headerfield to the top of the via-list
*/
void sipHeader::addVia(sipTagVia *v) {
	_via.push_front( v );
}

/* adds a "Via"-headerfield to the top of the via-list
*/
void sipHeader::addVia(const Socket::Protocol p, const char *ip, const int port) {
	_via.push_front( new sipTagVia(p, ip, port) );
}

void sipHeader::setViaList(list<sipTagVia *> via) {
	// the clear() might be problematic - only the pointers will be deleted!
	_via.clear();

	list<sipTagVia *>::iterator it_via = via.begin();
	while ( it_via != via.end() ) {
		_via.push_back( new sipTagVia(**it_via) );
		it_via++;
	}
}

/*
*/
void sipHeader::addRoute(sipTagRoute *r) {
	_route.push_front( r );
}

/*
*/
void sipHeader::addRoute(char *ip, int port) {
	_route.push_front( new sipTagRoute(ip, port) );
}

void sipHeader::setRouteList(list<sipTagRoute *> route) {
	// the clear() might be problematic - only the pointers will be deleted!
	_route.clear();

	list<sipTagRoute *>::iterator it_route = route.begin();
	while ( it_route != route.end() ) {
		_route.push_back( new sipTagRoute(**it_route) );
		it_route++;
	}
}

/*
void sipHeader::addRecordRoute(sipTagRecordRoute *r) {
	_recordRoute.push_front( r );
}

void sipHeader::addRecordRoute(char *ip, int port) {
	_recordRoute.push_front( new sipTagRecordRoute(ip, port) );
}

void sipHeader::setRecordRouteList(const list<sipTagRecordRoute *> rr) {
	_recordRoute = rr;
}
*/


void sipHeader::setContentType(char *s) {
	_contentType = opp_strdup( s );
}

void sipHeader::setContentLength(int i) {
	_contentLength = i;
}

void sipHeader::setRequire(char *s) {
	_require = opp_strdup( s );
}

void sipHeader::setAllow(int *allow) {
	for(int i = 0; i < MAX_ALLOW; i++)
		_allow[i] = allow[i];
};

void sipHeader::setSupported(int *supported) {
	for(int i = 0; i < MAX_SUPPORTED; i++)
		_supported[i] = supported[i];
};

void sipHeader::setAccept(int *accept) {
	for(int i = 0; i < MAX_ACCEPT; i++)
		_accept[i] = accept[i];
};

void sipHeader::setContentDisposition(int *disposition) {
	for(int i = 0; i < MAX_CONTENT_DISPOSITION; i++)
		_contentDisposition[i] = disposition[i];
};

void sipHeader::setContentLanguage(int *language) {
	for(int i = 0; i < MAX_CONTENT_LANGUAGE; i++)
		_contentLanguage[i] = language[i];
};

void sipHeader::setContentEncoding(int *encoding) {
	for(int i = 0; i < MAX_CONTENT_ENCODING; i++)
		_contentEncoding[i] = encoding[i];
};




void sipHeader::setExpires(int expires) {
	_expires = expires;
};

void sipHeader::setMaxForwards(const int i) {
	_maxForwards = i;
};



// --------------
// Get - Methoden
// --------------


sipAddress sipHeader::getUri() const {
	return _uri;
};

sipAddress sipHeader::getFrom() const {
	return _from;
}

sipAddress sipHeader::getTo() const {
	return _to;
}

unsigned int sipHeader::getContactCount() const {
	return _contact.size();
};

sipAddress *sipHeader::getContact(int i) const {
	if( (int)getContactCount() <= i )
		return NULL;
	else 
		return _contact[i];
};




stCallId sipHeader::getCallId() const {
	return _callId;
}

stCSeq sipHeader::getCSeq() const {
	return _cSeq;
}



/* returns the lists first via-element but does not remove it
*/
sipTagVia *sipHeader::getVia() const {
	if ( !_via.empty() ) {
		return _via.front();
	} else {
		return NULL;	// Wenn Liste leer, dann leeres Element
	}				// bergeben
}

/* returns the lists first via-element and then removes it
*/
sipTagVia *sipHeader::popVia() {
	if ( !_via.empty() ) {
		sipTagVia *v = _via.front();
		_via.erase( _via.begin() );
		return v;
	} else {
		return NULL;
	}
}

list<sipTagVia *> sipHeader::getViaList() const {
	return _via;
}

/* the same as getVia()
    returns the lists first Route-element but does not remove it
*/
sipTagRoute *sipHeader::getRoute() const {
	if ( !_route.empty() ) {
		return _route.front();
	} else {
		return NULL;	// Wenn Liste leer, dann leeres Element
	}			// bergeben
}

/* the same as popVia()
    returns the lists first Route-element and then removes it
*/
sipTagRoute *sipHeader::popRoute() {
	sipTagRoute *r = getRoute();
	if ( !_route.empty() ) {
		_route.erase( _route.begin() );
	}
	return r;
}

list<sipTagRoute *> sipHeader::getRouteList() const {
	return _route;
}


/* the same as getVia()
    returns the lists first Record-Route-element but does not remove it
*/
/*
sipTagRecordRoute *sipHeader::getRecordRoute() const {
	if ( !_recordRoute.empty() ) {
		return _recordRoute.front();
	} else {
		return NULL;	// Wenn Liste leer, dann leeres Element
	}			// bergeben
}
*/
/* the same as popVia()
    returns the lists first Record-Route-element and then removes it
*/
/*
sipTagRecordRoute *sipHeader::popRecordRoute() {
	sipTagRecordRoute *r = getRecordRoute();
	if ( !_recordRoute.empty() ) {
		_recordRoute.erase( _recordRoute.begin() );
	}
	return r;
}

list<sipTagRecordRoute *> sipHeader::getRecordRouteList() const {
	return _recordRoute;
}
*/


char *sipHeader::getContentType() const {
	return _contentType;
}

/* returns the messages content length :-)
*/
int sipHeader::getContentLength() const {
	return _contentLength;
}

char *sipHeader::getRequire() const {
	return _require;
}

int *sipHeader::getAllow() {
	return _allow;
};

int *sipHeader::getSupported() {
	return _supported;
};

int *sipHeader::getAccept() {
	return _accept;
};

int *sipHeader::getContentDisposition() {
	return _contentDisposition;
};

int *sipHeader::getContentLanguage() {
	return _contentLanguage;
};

int *sipHeader::getContentEncoding() {
	return _contentEncoding;
};



int sipHeader::getExpires() const {
	return _expires;
};

int sipHeader::getMaxForwards() const {
	return _maxForwards;
};
