/*!
	\file sipInnerPacket.h
	\author jAN bURGHARD
	\version 0.1
*/

#ifndef __SIPINNERPACKET_H__
#define __SIPINNERPACKET_H__

// OMNeT++-stuff
#include <omnetpp.h>

// SIP-stuff
#include "sipConstants.h"

/*  -------------------------------------------------
    Type Definitions
    -------------------------------------------------   */

//! Interfacepacket for communication between sub-modules
/*!
	\class sipInnerPacket sipInnerPacket.h
	\author Jan Burghard
	\version 0.1
	
	Interface message for internal SIP-Communication. E.G. between
	sipApplication and sipDialog.
*/
class sipInnerPacket: public cMessage {

public:
	enum sipMessageType {
		// MT = MessageType
		MT_UNDEF	= 0,
		MT_BYE		= 1,
		MT_CANCEL	= 2
	};


private:
	//! initializes the sipInnerPacket
	void _init();
	
	//! 
	sipMessageType _type;
	
public:
	//
	// Constructors, destructors,...
	//
	
	
	//! default constructor
	/*!
	constructor for named initialization
	\param name ...guess...
	*/
	sipInnerPacket(const char* name = "sipInnerPacket");
	
	/*!
	constructor for initialization with a packet-type parameter.
	*/
	sipInnerPacket(const sipMessageType type);
	
	/*!
	copy constructor
	\param sIfPkt packet to be copied
	*/
	sipInnerPacket(const  sipInnerPacket& sIfPkt );

	/*!
	assignement operator
	*/
	sipInnerPacket& operator=(const sipInnerPacket& sIfPkt );
	
	/*!
	duplicate method
	\return a copy of this
	*/
	virtual sipInnerPacket *dup() const {
		return new sipInnerPacket(*this); 
	}

	//! OMNeT++ method - returns the classname
	virtual const char *className() const { return "sipInnerPacket"; }
	
	//! OMNeT++ method - adds infos of this to buf
	virtual void info( char *buf );
	
	//! OMNeT++ method
	virtual void writeContents( ostream& os );
	
	//! returns the packet's type
	sipMessageType msgType();
	
};

#endif // __SIPINNERPACKET_H__
