/*!
	\file sipInterfacepacket.cc
	\author jAN bURGHARD
*/

#include "sipInterfacePacket.h"

// constructors
sipInterfacePacket::sipInterfacePacket()
	: cMessage("sipInterfacePacket")
{
	_init();
}

sipInterfacePacket::sipInterfacePacket(const char* name)
	: cMessage(name)
{
	_init();
}

// copy constructor
sipInterfacePacket::sipInterfacePacket( const sipInterfacePacket& sIfPkt ) 
	: cMessage()
{
	setName ( sIfPkt.name() );
	operator=( sIfPkt );
}

// private function
void sipInterfacePacket::_init()
{
	_action = sipInterfacePacket::SA_UNDEF;
	setKind( MK_SIPINTERFACEPACKET );
}

void sipInterfacePacket::_clear()
{
	_action = sipInterfacePacket::SA_UNDEF;
}

void sipInterfacePacket::_check()
{
	if (_action != sipInterfacePacket::SA_UNDEF)
		opp_error("Already initialized! Please don't reuse sipInterfacePackets.");
}

// assignment operator
sipInterfacePacket& sipInterfacePacket::operator=(const sipInterfacePacket& sIfPkt)
{
	cMessage::operator=(sIfPkt);
        _action = sIfPkt._action;

	return *this;
}

// output functions
void sipInterfacePacket::info( char *buf )
{
	cMessage::info( buf );
        sprintf(buf + strlen(buf), "  _action = %d ", (int) _action);
}

void sipInterfacePacket::writeContents(ostream& os)
{
	os << " _action = " << _action << endl;
}


void sipInterfacePacket::openPort(IN_Port port) {
	_check();
	
	_action = sipInterfacePacket::SA_OPEN_PORT;
	addPar("port") = port;
};


void sipInterfacePacket::timeoutRequest( sipRequest *request ) {
	_check();
	
	_action = sipInterfacePacket::SA_TIMEOUT_REQUEST;
	encapsulate( request );
};


void sipInterfacePacket::timeoutResponse( sipResponse *response ) {
	_check();
	
	_action = sipInterfacePacket::SA_TIMEOUT_RESPONSE;
	encapsulate( response );
};


void sipInterfacePacket::terminate() {
	_check();
	
	_action = sipInterfacePacket::SA_TERMINATE;
};


void sipInterfacePacket::forward( cMessage *msg ) {
	_check();
	
	_action = sipInterfacePacket::SA_FORWARD;
	encapsulate( msg );
};


void sipInterfacePacket::closeSession() {
	_check();
	
	_action = sipInterfacePacket::SA_CLOSE_SESSION;
};


void sipInterfacePacket::request( sipRequest *request ) {
	_check();
	
	_action = sipInterfacePacket::SA_REQUEST;
	encapsulate( request );
};

sipRequest *sipInterfacePacket::getRequest() {
	switch ( _action ) {
	case sipInterfacePacket::SA_REQUEST:
	case sipInterfacePacket::SA_ACK:
	case sipInterfacePacket::SA_TIMEOUT_REQUEST:
	case sipInterfacePacket::SA_FROM_SOCKET:
		return (sipRequest *)decapsulate();
		break;
		
	default:
		return NULL;
		break;
	}
};

void sipInterfacePacket::respond( sipResponse *response ) {
	_check();
	
	_action = sipInterfacePacket::SA_RESPONSE;
	encapsulate( response );
};

sipResponse *sipInterfacePacket::getResponse() {
	switch ( _action ) {
	case sipInterfacePacket::SA_RESPONSE:
	case sipInterfacePacket::SA_TIMEOUT_RESPONSE:
		return (sipResponse *)decapsulate();
		break;
		
	default:
		return NULL;
		break;
	}
};

void sipInterfacePacket::fromSocket( sipRequest *request ) {
	_check();
	
	_action = sipInterfacePacket::SA_FROM_SOCKET;
	encapsulate( request );
};

void sipInterfacePacket::ack( sipRequest *request ) {
	_check();
	
	_action = sipInterfacePacket::SA_ACK;
	encapsulate( request );
};

