/*!
	\file	sipResponse.cc
	\author	jAN bURGHARD
	\version 0.1
*/
#include "sipResponse.h"



// default constructor
sipResponse::sipResponse()
	: cMessage("sipResponse") 
{
	_init();
};

// copy constructor
sipResponse::sipResponse(const sipResponse& r)
	: cMessage()
{
	setName( r.name() );
	operator=( r );
};


//
sipResponse::sipResponse(const unsigned int statusCode) 
	: cMessage()
{
	_init();
	_statusCode = statusCode;
	
	char buf[10];
	sprintf( buf, "%3d", _statusCode );
	setName( buf ); 
	
};
	
	
//
sipResponse::sipResponse(const unsigned int stCode, const char *rPhrase)
	: cMessage()
{
	_init();

	_statusCode = stCode;
	_reasonPhrase = opp_strdup(rPhrase);
	
	char buf[10];
	sprintf( buf, "%3d", _statusCode );
	setName( buf ); 
};


//
sipResponse::~sipResponse() {
	delete _reasonPhrase;
	delete _sipVersion;
};


// initialization of basic settings
void sipResponse::_init() {
	// do not become owner of any messages...
	takeOwnership( true );

	//
	setKind( MK_SIPRESPONSE );
	
//	_destIp.setOwner( this );
//	_destPort.setOwner( this ); 
//	_destIp = IN_Addr();
//	_destPort = IN_Port();
	
	_sipVersion = opp_strdup( "2.0" );
	_statusCode = 999;
	_reasonPhrase = opp_strdup( "UNDEF" );
	
	_body.clear();
};


void sipResponse::info(char *buf) {
	cMessage::info( buf );
	sprintf( buf+strlen(buf), "%s %d '%s'", _sipVersion, _statusCode, _reasonPhrase);
};

void sipResponse::writeContents(ostream& os) {
	os << "(" << _sipVersion << "/" << _statusCode << ") " << _reasonPhrase << endl;
};

sipResponse& sipResponse::operator=(const sipResponse& r) {
	cMessage::operator=( r );

	_sipVersion	= opp_strdup( r._sipVersion );
	_statusCode	= r._statusCode;
	_reasonPhrase	= opp_strdup( r._reasonPhrase );
	
	_destIp.setAddr( r._destIp );
	_destPort.setPort( r._destPort );

	_header		= r._header;
	_body		= r._body;

//	_destIp.setOwner( this );
//	_destPort.setOwner( this );
	
	return *this;
};


void sipResponse::setDestination(const char *ip, const int port) {
	_destIp.setAddr( ip );
	_destPort.setPort( port );
};

	
/* ----------------------------------------------------------------------------
   --- set methods
   ----------------------------------------------------------------------------	*/
void sipResponse::setStatusCode(unsigned int status) {
        _statusCode = status;
};

void sipResponse::setReasonPhrase(char *reason) {
        _reasonPhrase = opp_strdup( reason );
};




/* ----------------------------------------------------------------------------
   --- get methods
   ----------------------------------------------------------------------------	*/
char *sipResponse::getSipVersion() const {
	return _sipVersion;
}

unsigned int sipResponse::getStatusCode() const {
	return _statusCode;
}

char *sipResponse::getReasonPhrase() const {
	return _reasonPhrase;
}


IN_Addr sipResponse::getDestIp() const {
	return _destIp;
};

IN_Port sipResponse::getDestPort() const {
	return _destPort;
};

// sipRequest::sip_request_method sipResponse::getMethod() const {
sipRequestMethod sipResponse::getMethod() const {
	return _header.getCSeq().method;
};

//
sipResponseHeader *sipResponse::header() {
	return &_header;
};

//
sipMessageBody *sipResponse::body() {
	return &_body;
};




char *sipResponse::responseKey() const {
	if ( _header.getVia() != NULL ) {
		return opp_concat( _header.getVia()->getBranch(),
		                   sipRequestMethodStr[ _header.getCSeq().method ] );
	}
	else {
		return sipRequestMethodStr[ _header.getCSeq().method ];
	}
};




// bool req: true if msg is request, else (msg is response) false
void sipResponse::writeMessageState() {
	cout << "XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX" << endl;
	cout << "XXXXXXXXXXXXXXXX MESSAGE STATE XXXXXXXXXXXXXXXXXXXXXXXXX" << endl;
	cout << "XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX" << endl;
	cout << "msg of class " << className() << endl;
	cout << "method: " << header()->getCSeq().method << endl;
	cout << "INVITE request" << endl;
	cout << "callID: " << header()->getCallId().id << endl;
	cout << "localTag: " << header()->getFrom().getTag();
	cout << " , remoteTag: " << header()->getTo().getTag() << endl;
	cout << "from: " << header()->getFrom().getIp();
	cout << " , to: " << header()->getTo().getIp()<< endl;
	cout << "request URI: " << header()->getUri().getIp() << endl;
//		cout << "via: " << inviteRequest->header()->getViaList() << endl;
	cout << "max forwards: " << header()->getMaxForwards() << endl;
	for(unsigned int i = 0; i < header()->getContactCount(); i++) {
		cout << "contact: " << header()->getContact( i )->getIp() << endl;
	}
	cout << "status: " << getStatusCode() << getReasonPhrase() << endl;
	cout << "XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX" << endl;
	cout << "XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX" << endl;
	cout << "XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX" << endl;
};
