/*!
	\file	sipResponse.h
	\author	jAN bURGHARD
	\version 0.1
*/
#ifndef __SIPRESPONSE_H
#define __SIPRESPONSE_H

// for gcc3.x
using namespace std;

// OMNeT++-stuff
#include <omnetpp.h>

// SIP-stuff
#include "sipResponseHeader.h"
#include "sipMessageBody.h"
#include "sipConstants.h"
#include "sipRequest.h"
#include "sipTags.h"


//! SIP-Response
/*!
	\class sipResponse sipResponse.h
	\author Jan Burghard
	
	This class represents a response message of the SIP
*/
class sipResponse : public cMessage {

private:
	/* ----------------------------------------------------------------------------
	   --- Private class variables
	   ----------------------------------------------------------------------------	*/
	//! protocol version
	char		*_sipVersion;
	
	//! three digit returncode
	unsigned int	_statusCode;
	
	//! human readable returncode
	char 		*_reasonPhrase;

	//! next-hop destination
	IN_Addr	_destIp;
	//! next-hop port
	IN_Port	_destPort;
        
	//! Message header
	sipResponseHeader	_header;

	/* ----------------------------------------------------------------------------
	   --- Private class functions
	   ----------------------------------------------------------------------------	*/
	//! initialization of basic class settings
	void _init();

public:
	/* ----------------------------------------------------------------------------
	   --- Public class variables
	   ----------------------------------------------------------------------------	*/
	//! Message body
	sipMessageBody		_body;


	/* ----------------------------------------------------------------------------
	   --- Constructors / Destructor
	   ----------------------------------------------------------------------------	*/
	//! default constructor
	sipResponse();
	
	//! copy constructor
	sipResponse(const sipResponse& r);
	
	//! constructor for fast execution
	sipResponse(const unsigned int statusCode);
	
	/*!
	constructor that initializes a reponse with ...
	*/
	sipResponse(const unsigned int stCode, const char *rPhrase);

	//! destructor
	~sipResponse();
	
	/* ----------------------------------------------------------------------------
	   --- OMNeT++ specific functions
	   ----------------------------------------------------------------------------	*/
	//! returns class name string
	virtual const char *className() const { return "sipResponse"; }
	//! duplicates the object by creating an exact copy (uses copy constructor)
	virtual sipResponse *dup() const { return new sipResponse(*this); }
	//! returns a one-line info about object contents
	virtual void info(char *buf);
	//! write a more detailed report about the object into a file
	virtual void writeContents(ostream& os);
	//! the assignement operator (copies object contents from another object)
	sipResponse& operator=(const sipResponse& r);
	
	
	/* ----------------------------------------------------------------------------
	   --- get methods
	   ----------------------------------------------------------------------------	*/
	   
	//! returns the sipVersion
	char *getSipVersion() const;
	//! returns the responses statuscode
	unsigned int getStatusCode() const;
	//! returns the explaining reason-phrase
	char *getReasonPhrase() const;
	//! returns the responses destination ip-address
	IN_Addr getDestIp() const;
	//! returns the responses destination port
	IN_Port getDestPort() const;
	//! returns the sipRequestMethod to which this response was created
	sipRequestMethod getMethod() const;
	
	//! returns a pointer to the sipRequestHeader
	sipResponseHeader *header();
	
	//! returns a pointer to the sipMessageBody
	sipMessageBody *body();
	//! returns the response-key
	/*!
	this key is created with the branch-parameter of the topmost Via-headerfiled
	and the request-method of the CSeq-headerfield.
	*/
	char *responseKey() const;
	
	//! to set the message's statuscode
        void setStatusCode(unsigned int status);
	//! to set the reason-phrase
        void setReasonPhrase(char *reason);
        

	//! for debugging issues
	void writeMessageState();

	//! to set the message's destination ip and port
	void setDestination(const char *ip, const int port);
	
};

#endif
