/*
        file: sipStructs.h
        author: Susann Geisler
        RFC 3261
        
        this file includes all structs which are 
        used in sip headers
*/


#ifndef SIPSTRUCTS_H 
#define SIPSTRUCTS_H 

enum transportParam {udp=1, tcp=2, sctp=3, tls=4};

enum userParam {phone=1, ip=2};

enum methodParam {INVITE=1,
                  ACK=2, CANCEL=3, BYE=4, OPTIONS=5,
		  REGISTER=6};

/*
enum lowalpha {a, b, c, d, e, f, g, h, i,
               j, k, l, m, n, o, p, q, r,
               s, t, u, v, w, x, y, z};
	       
enum upalpha {A, B, C, D, E, F, G, H, I, J,
              K, L, M, N, O, P, Q, R, S, T,
	      U, V, W, X, Y, Z};
*/      
//enum sipDigit {0, 1, 2, 3, 4, 5, 6, 7, 8, 9};
//enum alphanum {lowalpha, upalpha, digit};


const int DEFAULT_PORT = 5060;
const int DEFAULT_TTL = 75;
const enum transportParam DEFAULT_TRANSPORT_PROTOCOL = udp;
const enum userParam DEFAULT_USER_PARAM = ip;
const enum methodParam DEFAULT_METHOD = INVITE;


struct hostname {
        /* 
         domainlabel = alphanum | alphanum*(alphanum|"-")alphanum
        */
        char *domainlabel;
        /* 
         toplabel = alpha | alpha*(alphanum|"-")alphanum
        */
        char *toplabel;
};

struct ipv4address {
        unsigned char firstNo; // possible values: 0 - 255
        unsigned char secondNo;
        unsigned char thirdNo;
        unsigned char fourthNo;
};

/* either hostname or ipv4address 
   ipv6 not implemented yet 
*/
union address {
        struct hostname host;
//        struct ipv4address ip;
	char *ip;
};

union useraddr {
        char *usr;
        int telephonesubscriber;
};

struct userinfo {
        union useraddr user; /* user name or telephone-subscriber
	                     */
	char *password;
};

struct hostport {
        union address addr;
        /*      default values of the port:
                5060 for sip for udp, tcp or sctp
                5061 for sip for tls over tcp and sips over tcp 
        */
	int port; 
};

struct urlParameters {
	enum transportParam transport;
	enum userParam user;
	enum methodParam method;
	int ttlParam;
	struct ipv4address maddrParam;
	char *otherParam;
};

struct headers {
	char *hname;
	char *hvalue;
};

#endif
