/*
	\file	sipTags.cc
	\author Susann Geisler	
	\author	jAN bURGHARD
	\version 0.1
*/

#include "sipTags.h"






/* ----------------------------------------------------------------------
   class sipAddress
   ---------------------------------------------------------------------- */

// initialization of elements
void sipAddress::_init() {
	_user		= NULL;
	_password	= NULL;
	_telesub	= 0;
	_domainlabel	= NULL;
	_toplabel	= NULL;
	_host		= NULL;
	_ip		= NULL;
	_port		= 0;
	_transportProt	= Socket::PROTO_UNDEF;
	_userparam	= sipAddress::UNDEF;
	_requestmethod	= 0;
	_ttlparam	= 0;
	_maddrparam	= NULL;
        _tag            = NULL;
};

// default constructor
sipAddress::sipAddress() 
{
	_init();
/*	
	_ip   = opp_strdup( "127.0.0.1" );
	_host = opp_strdup( "localhost" );
	_port = 5060;
*/
};

// copy constructor
sipAddress::sipAddress(const sipAddress& a)
{
	operator=( a );
};

//
sipAddress::sipAddress(const char *user, const char *host)
{
	_init();
	
	_user = opp_strdup(user);
	_host = opp_strdup(host);
	_port = 0;
};

//
sipAddress::sipAddress(const char *ip, const int port)
{
	_init();
	
	_ip   = opp_strdup(ip);
	_port = port;
};

//
sipAddress::sipAddress(const char *user, const char *ip, const int port)
{
	_init();

	_user	= opp_strdup( user );
	_ip	= opp_strdup( ip );
	_port	= port;
};


//
sipAddress::~sipAddress() {
	delete _user;
	delete _password;
	delete _domainlabel;
	delete _toplabel;
	delete _host;
	delete _ip;
	delete _maddrparam;
};


//
sipAddress& sipAddress::operator=(const sipAddress& a) {
	_user		= opp_strdup( a._user );
	_password	= opp_strdup( a._password );
	_telesub	= a._telesub;
	_domainlabel	= opp_strdup( a._domainlabel );
	_toplabel	= opp_strdup( a._toplabel );
	_host		= opp_strdup( a._host );
	_ip		= opp_strdup( a._ip );
	_port		= a._port;
	_transportProt	= a._transportProt;
	_userparam	= a._userparam;
	_requestmethod	= a._requestmethod;
	_ttlparam	= a._ttlparam;
	_maddrparam	= opp_strdup( a._maddrparam );
	_tag            = opp_strdup(a._tag);
	
	return *this;
};


bool sipAddress::equals(const sipAddress& a) {
	return 0 == opp_strcmp( _user, a._user )
		&& 0 == opp_strcmp( _ip, a._ip )
		&& 0 == opp_strcmp( _host, a._host )
		&& _port == a._port;
};


char *sipAddress::toChar() const {
	char *s = "";
	
	// Username
	if( _user != NULL ) s = opp_concat(_user, "@");
	// Host / IP
	if( _host != NULL ) {
		s = opp_concat( s, _host );
	} else {
		s = opp_concat( s, _ip );
	}
	// Port
	char strbuf[20];
	sprintf( strbuf, "%i", _port );
	s = opp_concat( s, ":", strbuf );


	return opp_strdup( s );
};

//
void sipAddress::setUser(const char *s) {
	_user = opp_strdup( s );
}

//
void sipAddress::setPassword(const char *s) {
	_password = opp_strdup( s );
}

//
void sipAddress::setTelesub(const int i) {
	_telesub = i;
}

// 
void sipAddress::setDomainlabel(const char *s) {
	_domainlabel = opp_strdup( s );
}

// 
void sipAddress::setToplabel(const char *s) {
	_toplabel = opp_strdup( s );
}

// 
void sipAddress::setHost(const char *s) {
	_host = opp_strdup( s );
}

//
void sipAddress::setIp(const char *s) {
	_ip = opp_strdup( s );
}

//
void sipAddress::setPort(const int i) {
	_port = i;
}

//
void sipAddress::setTransportProtocol(const Socket::Protocol s) {
	_transportProt = s;
}

//
void sipAddress::setUserParam(const sipAddress::userParam userp) {
	_userparam = userp;
};

//
void sipAddress::setMethodParam(const int method) {
	_requestmethod = method;
};

//
void sipAddress::setTtlParam(const int i) {
	_ttlparam = i;
}

//
void sipAddress::setMaddrParam(const char *ip) {
	_maddrparam = opp_strdup( ip );
};

//
void sipAddress::setTag(char *tag) {
	_tag = opp_strdup( tag );
};

// 
char *sipAddress::getUser() const {
	return _user;
}

// 
char *sipAddress::getPassword() const {
	return _password;
}

// 
int sipAddress::getTelesub() const {
	return _telesub;
}

//
char *sipAddress::getDomainlabel() const {
	return _domainlabel;
}

//
char *sipAddress::getToplabel() const {
	return _toplabel;
}

//
char *sipAddress::getHost() const {
	return _host;
}

//
char *sipAddress::getIp() const {
	return _ip;
}

// 
int sipAddress::getPort() const {
	return _port;
}

// 
Socket::Protocol sipAddress::getTransportProtocol() const {
	return _transportProt;
};

// 
sipAddress::userParam sipAddress::getUserParam() const {
	return _userparam;
};

// 
int sipAddress::getMethodParam() const {
	return _requestmethod;
}

// 
int sipAddress::getTtlParam() const {
	return _ttlparam;
}

//
char *sipAddress::getMaddrParam() const {
	return _maddrparam;
}

//
char *sipAddress::getTag() {
	return _tag;
};

//
char *sipAddress::toString() const {
	return opp_concat(_user, "@", _ip);
}


/* ----------------------------------------------------------------------
   class sipTagVia
   ---------------------------------------------------------------------- */
void sipTagVia::_init() {
	_protocol	= Socket::PROTO_UNDEF;
	_ip		= NULL;
	_hostname	= NULL;
	_port		= 0;
	_paramHidden	= false;
	_paramTtl	= 0;
	_paramMaddr	= NULL;
	_paramReceived	= NULL;
	_paramBranch	= NULL;
};

// default constructor
sipTagVia::sipTagVia( const char *ip = "0.0.0.0" )
{
	_init();
	setIp( ip );
};

// copy constructor
sipTagVia::sipTagVia(const sipTagVia& v)
{
	_init();
	operator=( v );
};

//
sipTagVia::sipTagVia(const Socket::Protocol p, const char *ip, const int port)
{ 
	_init();
	_protocol	= p;
	_ip		= opp_strdup( ip );
	_port		= port;
};

//
sipTagVia::~sipTagVia() {
	delete _ip;
	delete _hostname;
	delete _paramMaddr;
	delete _paramReceived;
	delete _paramBranch;
};

// = operator
sipTagVia& sipTagVia::operator=(const sipTagVia& via) {
	_protocol	= via._protocol;
	_ip		= opp_strdup( via._ip);
	_hostname	= opp_strdup( via._hostname);
	_port		= via._port;
	_paramHidden	= via._paramHidden;
	_paramTtl	= via._paramTtl;
	_paramMaddr	= opp_strdup( via._paramMaddr);
	_paramBranch	= opp_strdup( via._paramBranch);
	
	if ( via._paramReceived != NULL ) {
		_paramReceived = via._paramReceived->dup();
	} // else { _paramReceived = NULL; }
	
	return *this;
};

void sipTagVia::setProtocol(const Socket::Protocol p) {
	_protocol = p;
};
void sipTagVia::setIp(const char *s) {
	_ip = opp_strdup(s);
};
void sipTagVia::setPort(const int i) {
	_port = i;
};
void sipTagVia::setBranch(const char *s) {
	_paramBranch = opp_strdup(s);
};
void sipTagVia::setReceived(const char *ip, const int port) {
	delete _paramReceived;
	_paramReceived = new sipAddress( ip, port );
};
	

Socket::Protocol sipTagVia::getProtocol() const {
	return _protocol;
};
char *sipTagVia::getIp() const {
	return _ip;
};
int sipTagVia::getPort() const {
	return _port;
};
char *sipTagVia::getBranch() const {
	return _paramBranch;
};

sipAddress *sipTagVia::getReceived() const {
	return _paramReceived;
};

char *sipTagVia::toString() const {
	char *s;
	s = opp_concat( "Via: ", getIp() );
	
	if ( 0 != opp_strcmp("", _paramBranch) ) {
		s = opp_concat( s, ";branch=", _paramBranch );
	}
	return s;
};









/* ----------------------------------------------------------------------
   class sipTagRoute
   ---------------------------------------------------------------------- */
// default constructor
sipTagRoute::sipTagRoute()
{
	_ip = NULL;
	_hostname = NULL;
};

//
sipTagRoute::sipTagRoute(const char *ip, const int port)
{ 
	_ip = opp_strdup( ip );
	_port = port;
	_hostname = NULL;
};


// copy constructor
sipTagRoute::sipTagRoute(const sipTagRoute& r)
{
	operator=( r );
};

//
sipTagRoute::~sipTagRoute() {
	delete _ip;
	delete _hostname;
};

// = operator
sipTagRoute& sipTagRoute::operator=(const sipTagRoute& route) {
	_ip		= opp_strdup( route._ip );
	_hostname	= opp_strdup( route._hostname );
	_port		= route._port;
	return *this;
};

void sipTagRoute::setIp(char *s) {
	_ip = opp_strdup( s );
};
void sipTagRoute::setPort(int i) {
	_port = i;
};


char *sipTagRoute::getIp() const {
	return _ip;
};
int sipTagRoute::getPort() const {
	return _port;
};

char *sipTagRoute::toString() const {
	return "Not implemented yet!";
};







/* ----------------------------------------------------------------------
   class sipTagRecord-Route
   ---------------------------------------------------------------------- */
// default constructor
sipTagRecordRoute::sipTagRecordRoute() 
{
	_lrFlag = false;
	_ip = NULL;
	_hostname = NULL;
};

//
sipTagRecordRoute::sipTagRecordRoute(const char *ip, const int port) 
{
	_ip = opp_strdup(ip);
	_port = port;
	_lrFlag = false;
	_hostname = NULL;
};

// copy constructor
sipTagRecordRoute::sipTagRecordRoute(const sipTagRecordRoute& r)
{
	operator=( r );
};


//
sipTagRecordRoute::~sipTagRecordRoute() {
	delete _ip;
	delete _hostname;
};


sipTagRecordRoute& sipTagRecordRoute::operator=(const sipTagRecordRoute& rroute) {
	_ip		= opp_strdup( rroute._ip );
	_hostname	= opp_strdup( rroute._hostname );
	_port		= rroute._port;
	_lrFlag		= rroute._lrFlag;
	return *this;
};

void sipTagRecordRoute::setIp(char *s) {
	_ip = opp_strdup( s );
};
void sipTagRecordRoute::setPort(int i) {
	_port = i;
};
void sipTagRecordRoute::setLr(bool b) {
	_lrFlag = b;
};

char *sipTagRecordRoute::getIp() const {
	return _ip;
};
int sipTagRecordRoute::getPort() const {
	return _port;
};
bool sipTagRecordRoute::getLr() const {
	return _lrFlag;
};

char *sipTagRecordRoute::toString() const {
	return "Not implemented yet!";
};







