/*!
	\file	simpleRouting.h
	\author	jAN bURGHARD
	\version 0.1
	
*/
#ifndef __SIMPLEROUTING_H
#define __SIMPLEROUTING_H

// for gcc3.x
using namespace std;

// C++ stuff
#include <string>
#include <vector>

// OMNeT++ stuff
#include <omnetpp.h>

//! a simple routing mechanism
/*!
	\class simpleRouting simpleRouting.h
	\author Jan Burghard
	
	This simple routing class figures out the "next hop" ip.
	For a given ip-address
	(a.b.c.d | d > 1) it returns (a.b.c.1) as "next hop" proxy.
*/
class simpleRouting {

public:
	/*!
	this class stores an IPv4 address as 4 int numbers
	*/
	class quartet {
	public:
		int digit[4];
		
		/*!
		compares two ip-addresses
		\return amount of equal ip sets. Compared from left to right.
		*/
		int cmp(int quartet[4]) {
			int i = 0;
			while ( (i < 4) && (digit[i] == quartet[i]) ) { i++; }
			return i;
		}
	};

private:
	/*! vector of local ip-addresses                                     */
	vector<quartet> _localIp;
	/*! RTP port number                                                  */
	int _port;
	
	//! converts an ip-address string to native form
	/*!
	\param ip the ip-address as char*
	\param digits four int-values where the ip-address will be stored in
	*/
	void _splitIp(const char *ip, int digits[4]);
	
public:
	//! default constructor
	/*!
	initializes _port with 5060.
	*/
	simpleRouting();

	//! add a local ip
	/*!
	\param ip as char*
	*/
	void addLocalIp(const char *ip);
	
	//! sets the local port
	void setLocalPort(int port);

	//! returns next hops ip-address
	/*!
	if the ip's aren't in the same subnet the next-hop-ip is
	the remote host with the ip 1
	\return ip-address as char*
	*/
	char *nextHopIp(const char *ip);
	
	//! at the moment it is for all ip's the same
	/*!
	\return _port
	*/
	int nextHopPort(const char *ip);
	
	//! checks if the given ip is one of the local ip's
	/*!
	\param ip as char*
	\return bool-value
	*/
	bool isInLocals(const char *ip);
};

#endif
