/*!
	\file	sipLocationService.cc
	\author	jAN bURGHARD
	\version 0.1
	
*/
#include "sipLocationService.h"



sipLocationRecord::sipLocationRecord( const sipAddress *address ) :
	cObject("SIP Location Record") {	
	_addr_of_record = address->dup();
};

sipLocationRecord::~sipLocationRecord() {	
	for(unsigned int i = 0; i < _mapAddresses.size(); i++) {
		delete _mapAddresses[i];
	}
	_mapAddresses.clear();
	
	delete _addr_of_record;	
};

int sipLocationRecord::addMapAddress( sipAddress *address ) {
	_mapAddresses.push_back( address->dup() );
	return _mapAddresses.size();
};

unsigned int sipLocationRecord::mapCount() {
	return _mapAddresses.size();
};

sipAddress *sipLocationRecord::mapAddress( unsigned int index ) {
	if ( index < mapCount() ) {
		return _mapAddresses[index];
	} else {
		return NULL;
	}
};

int sipLocationRecord::isMapped( sipAddress *address ) {
	for(unsigned int i = 0; i < mapCount(); i++) {
		if ( mapAddress(i) == address ) {
			return i;
		}
	}
	return -1;
}

sipAddress *sipLocationRecord::addrOfRecord() const {
	return _addr_of_record;
};




// returns a one-line info about object contents
void sipLocationRecord::info(char *buf) {
	cObject::info( buf );
	sprintf( buf+strlen(buf), " sipLocationRecord");
};

// write a more detailed report about the object into a file
void sipLocationRecord::writeContents(ostream& os) {
	os << " sipLocationRecord" << endl;
};

// the assignement operator (copies object contents from another object)
sipLocationRecord& sipLocationRecord::operator=(const sipLocationRecord& slr) {
	cObject::operator=( slr );
	
	// copy addr_of_record 
	_addr_of_record = slr._addr_of_record->dup();
	
	// empty _mapAddresses and copy them from slr
	while( !_mapAddresses.empty() ) {
		delete _mapAddresses.front();
		_mapAddresses.erase( _mapAddresses.begin() );
	}
	for(unsigned int i = 0; i < slr._mapAddresses.size(); i++) {
		_mapAddresses.push_back( slr._mapAddresses[i]->dup() );
	}	
	
	return *this;
};










sipLocationService::sipLocationService() :
	cObject("SIP Location Service")
{
	_routingModule = NULL;
	_debug = true;
};



int sipLocationService::addMapAddress(const sipAddress *address, const sipAddress *map) {
	sipLocationRecord *r = NULL;
	
	for(unsigned int i = 0; i < _entrys.size(); i++) {
		r = _entrys[i];
	}
	if ( r == NULL ) {
		_entrys.push_back( new sipLocationRecord(address->dup()) );
	} else {
	
	}
	return _entrys.size();
};


sipResponse *sipLocationService::handleRegister( sipRequest *request ) {
	// constructing response	
	sipResponse *response = new sipResponse();
	// Copy Request-Header
	response->header()->setViaList( request->header()->getViaList() );
	response->header()->setCSeq( request->header()->getCSeq() );
	response->header()->setTo( request->header()->getTo() );
	response->header()->setFrom( request->header()->getFrom() );

	// we take the destination from the via-tag
	sipAddress *received = request->header()->getVia()->getReceived();
	response->setDestination( received->getIp(), received->getPort() );


/*	
      1. The registrar inspects the Request-URI to determine whether it
         has access to bindings for the domain identified in the
         Request-URI.  If not, and if the server also acts as a proxy
         server, the server SHOULD forward the request to the addressed
         domain, following the general behavior for proxying messages
         described in Section 16.

      2. To guarantee that the registrar supports any necessary
         extensions, the registrar MUST process the Require header field
         values as described for UASs in Section 8.2.2.

      3. A registrar SHOULD authenticate the UAC.  Mechanisms for the
         authentication of SIP user agents are described in Section 22.
         Registration behavior in no way overrides the generic
         authentication framework for SIP.  If no authentication
         mechanism is available, the registrar MAY take the From address
         as the asserted identity of the originator of the request.

      4. The registrar SHOULD determine if the authenticated user is
         authorized to modify registrations for this address-of-record.
         For example, a registrar might consult an authorization
         database that maps user names to a list of addresses-of-record
         for which that user has authorization to modify bindings.  If
         the authenticated user is not authorized to modify bindings,
         the registrar MUST return a 403 (Forbidden) and skip the
         remaining steps.

         In architectures that support third-party registration, one
         entity may be responsible for updating the registrations
         associated with multiple addresses-of-record.

      5. The registrar extracts the address-of-record from the To header
         field of the request.  If the address-of-record is not valid
         for the domain in the Request-URI, the registrar MUST send a
         404 (Not Found) response and skip the remaining steps.  The URI
         MUST then be converted to a canonical form.  To do that, all
         URI parameters MUST be removed (including the user-param), and
         any escaped characters MUST be converted to their unescaped
         form.  The result serves as an index into the list of bindings.

      6. The registrar checks whether the request contains the Contact
         header field.  If not, it skips to the last step.  If the
         Contact header field is present, the registrar checks if there
         is one Contact field value that contains the special value "*"
         and an Expires field.  If the request has additional Contact
         fields or an expiration time other than zero, the request is
         invalid, and the server MUST return a 400 (Invalid Request) and
         skip the remaining steps.  If not, the registrar checks whether
         the Call-ID agrees with the value stored for each binding.  If
         not, it MUST remove the binding.  If it does agree, it MUST
         remove the binding only if the CSeq in the request is higher
         than the value stored for that binding.  Otherwise, the update
         MUST be aborted and the request fails.

      7. The registrar now processes each contact address in the Contact
         header field in turn.  For each address, it determines the
         expiration interval as follows:

         -  If the field value has an "expires" parameter, that value
            MUST be taken as the requested expiration.

         -  If there is no such parameter, but the request has an
            Expires header field, that value MUST be taken as the
            requested expiration.

         -  If there is neither, a locally-configured default value MUST
            be taken as the requested expiration.

         The registrar MAY choose an expiration less than the requested
         expiration interval.  If and only if the requested expiration
         interval is greater than zero AND smaller than one hour AND
         less than a registrar-configured minimum, the registrar MAY
         reject the registration with a response of 423 (Interval Too
         Brief).  This response MUST contain a Min-Expires header field
         that states the minimum expiration interval the registrar is
         willing to honor.  It then skips the remaining steps.

         Allowing the registrar to set the registration interval
         protects it against excessively frequent registration refreshes
         while limiting the state that it needs to maintain and
         decreasing the likelihood of registrations going stale.  The
         expiration interval of a registration is frequently used in the
         creation of services.  An example is a follow-me service, where
         the user may only be available at a terminal for a brief
         period.  Therefore, registrars should accept brief
         registrations; a request should only be rejected if the
         interval is so short that the refreshes would degrade registrar
         performance.
         For each address, the registrar then searches the list of
         current bindings using the URI comparison rules.  If the
         binding does not exist, it is tentatively added.  If the
         binding does exist, the registrar checks the Call-ID value.  If
         the Call-ID value in the existing binding differs from the
         Call-ID value in the request, the binding MUST be removed if
         the expiration time is zero and updated otherwise.  If they are
         the same, the registrar compares the CSeq value.  If the value
         is higher than that of the existing binding, it MUST update or
         remove the binding as above.  If not, the update MUST be
         aborted and the request fails.

         This algorithm ensures that out-of-order requests from the same
         UA are ignored.

         Each binding record records the Call-ID and CSeq values from
         the request.

         The binding updates MUST be committed (that is, made visible to
         the proxy or redirect server) if and only if all binding
         updates and additions succeed.  If any one of them fails (for
         example, because the back-end database commit failed), the
         request MUST fail with a 500 (Server Error) response and all
         tentative binding updates MUST be removed.

      8. The registrar returns a 200 (OK) response.  The response MUST
         contain Contact header field values enumerating all current
         bindings.  Each Contact value MUST feature an "expires"
         parameter indicating its expiration interval chosen by the
         registrar.  The response SHOULD include a Date header field.
*/
	// ------
	// step 5
	// ------
	sipAddress *addr_of_record = request->getTo().dup();
	if ( _debug ) ev << addr_of_record->toChar() << endl;
	
	// test responsibility
	// either we've no routing module and so have to skip this test
	// or the "next-hop"-ip for the "addr_of_record"-ip hast to be one
	// of the "local"-ips
	if ( (_routingModule != NULL) &&
	     (!_routingModule->isInLocals( addr_of_record->getIp())) )
	{
		delete addr_of_record;
		response->setStatusCode( 404 );
		response->setReasonPhrase( "Not found" );
		return response;
	}
	
	sipLocationRecord *record = getRecord( addr_of_record );
	if ( record == NULL ) {
		record = new sipLocationRecord( addr_of_record );
		_entrys.push_back( record );
	}
	
	// ------
	// step 6
	// ------
	// TBD
	
	// ------
	// step 7
	// ------
	sipAddress *mapAddress;
	for(unsigned int i = 0; i < request->header()->getContactCount(); i++) {
		mapAddress = request->header()->getContact( i );
		if ( _debug ) ev << " -> mapping to:" << mapAddress->toChar() << endl;
		
		if ( record->isMapped(mapAddress) == -1 ) {
			record->addMapAddress( mapAddress );
		}
	}
	
	// ------
	// step 8
	// ------
	for(unsigned int i = 0; i < record->mapCount(); i++) {
		ev << record->addrOfRecord()->toChar() << " mapped to " << record->mapAddress(i)->toChar() << endl;
		response->header()->addContact( record->mapAddress(i)->dup() );
	}
	
	// respond with 200
	response->setStatusCode( 200 );
	
	return response;
};

sipLocationRecord *sipLocationService::getRecord( const sipAddress *address ) {
	for(unsigned int i = 0; i < _entrys.size(); i++) {
		if ( _entrys[i]->addrOfRecord()->equals(*address) ) {
			return _entrys[i];
		}
	}
	return NULL;
};



// returns a one-line info about object contents
void sipLocationService::info(char *buf) {
	cObject::info( buf );
	sprintf( buf+strlen(buf), " sipLocationService");
};

// write a more detailed report about the object into a file
void sipLocationService::writeContents(ostream& os) {
	os << " sipLocationService" << endl;
};

// the assignement operator (copies object contents from another object)
sipLocationService& sipLocationService::operator=(const sipLocationService& sls) {
	cObject::operator=( sls );
	
	_debug = sls._debug;
	
	// remove _entrys and copy them from sls
	while( !_entrys.empty() ) {
		delete _entrys.front();
		_entrys.erase( _entrys.begin() );
	}
	for(unsigned int i = 0; i < sls._entrys.size(); i++) {
		_entrys.push_back( sls._entrys[i]->dup() );
	}
	
	
	return *this;
};
